{% extends "base.html" %}
{% load crispy_forms_tags %}
{% load i18n %}

{% block title %}
{% translate "Request working hours" %}
{% endblock %}

{% block content %}
    <h1 class="page-header">
        {% trans "Request working hours" %}
    </h1>
    {% crispy form %}
{% endblock %}
