{% extends "base.html" %}
{% load crispy_forms_filters %}
{% load i18n %}

{% block title %}
    {% trans "Reset password" %}
{% endblock %}

{% block content %}
    <h2>{% trans "Reset password" %}</h2>
    <form method="post">
        {% csrf_token %}
        {{ form|crispy }}
        <button class="btn btn-primary" type="submit">{% translate "Reset password" %}</button>
    </form>
{% endblock %}
