{% extends "base.html" %}
{% load i18n %}

{% block title %}
    {% translate "Reset another user's password" %}
{% endblock %}

{% block content %}
    <div class="page-header">
        <h1>{% translate "Reset another user's password" %}</h1>
    </div>
    <p>{% blocktranslate with name=userprofile.get_full_name %}
        Are you sure that you want to reset the password of the user "{{ name }}"?
    {% endblocktranslate %}</p>
    <form method="post">
        {% csrf_token %}
        <input type="hidden" name="confirm" value="true">
        <a role="button" class="btn btn-secondary"
           href="{% url "core:userprofile_edit" userprofile.pk %}">{% translate "Back" %}</a>
        <button type="submit" class="btn btn-danger">{% translate "Reset password" %}</button>
    </form>
{% endblock %}