{% extends "base.html" %}
{% load i18n %}
{% load static %}

{% block title %}
    {% trans "Delete group" %}
{% endblock %}

{% block content %}
    <div class="page-header">
        <h1>{% trans "Delete group" %}</h1>
    </div>

    <form method="post">{% csrf_token %}
        <p>{% blocktrans %}Are you sure you want to delete the group "{{ group }}"?{% endblocktrans %}</p>
        <a role="button" class="btn btn-secondary" href="{% url "core:group_list" %}">{% trans "Back" %}</a>
        <button type="submit" class="btn btn-danger">{% trans "Delete" %}</button>
    </form>

{% endblock %}
