{% extends "base.html" %}
{% load i18n %}

{% block title%}
    offline
{% endblock %}

{% block content %}
    <h1>{% translate "No connection" %}</h1>
    <p>{% translate "You are currently offline. Please check your internet connection and try again." %}</p>
{% endblock %}
