{% extends "core/settings/settings_base.html" %}
{% load crispy_forms_filters %}
{% load i18n %}

{% block settings_content %}
    <div class="page-header">
        {% if qualification %}
            <h2>{% trans "Edit qualification" %}</h2>
        {% else %}
            <h2>{% trans "Create new qualification" %}</h2>
        {% endif %}
    </div>

    {% if qualification.is_imported %}
        <div class="alert alert-warning" role="alert">
            {% blocktranslate %}
                This qualification was imported. We advise to not change the UUID and meaning of this qualification.
            {% endblocktranslate %}
        </div>
    {% endif %}

    <form method="post">
        {% csrf_token %}
        {{ form|crispy }}
        <a role="button" class="btn btn-secondary"
           href="{% url "qualification_management:settings_qualification_list" %}">{% translate "Back" %}</a>
        <input type="submit" class="btn btn-primary" value="Save">
    </form>
{% endblock %}
