# Generated by Django 3.1.6 on 2021-02-07 12:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0007_auto_20210110_0050"),
    ]

    operations = [
        migrations.AlterModelTable(
            name="consequence",
            table="consequence",
        ),
        migrations.AlterModelTable(
            name="qualification",
            table="qualification",
        ),
        migrations.AlterModelTable(
            name="qualificationcategory",
            table="qualificationcategory",
        ),
        migrations.AlterModelTable(
            name="qualificationgrant",
            table="qualificationgrant",
        ),
        migrations.AlterModelTable(
            name="userprofile",
            table="userprofile",
        ),
        migrations.AlterModelTable(
            name="workinghours",
            table="workinghours",
        ),
    ]
