# Generated by Django 3.2.8 on 2021-10-27 20:03

import django.db.models.manager
from django.db import migrations, models

import ephios.core.models.users


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0012_rename_included_qualifications_qualification_includes"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="event",
            options={
                "base_manager_name": "all_objects",
                "default_manager_name": "all_objects",
                "permissions": [("view_past_event", "Can view past events")],
                "verbose_name": "event",
                "verbose_name_plural": "events",
            },
        ),
        migrations.AlterModelOptions(
            name="userprofile",
            options={
                "base_manager_name": "all_objects",
                "default_manager_name": "all_objects",
                "verbose_name": "user profile",
                "verbose_name_plural": "user profiles",
            },
        ),
        migrations.AlterModelManagers(
            name="event",
            managers=[
                ("all_objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="userprofile",
            managers=[
                ("all_objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AddField(
            model_name="userprofile",
            name="is_visible",
            field=models.BooleanField(default=True, verbose_name="Visible"),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="expires",
            field=ephios.core.models.users.ExpirationDateField(
                blank=True, null=True, verbose_name="expiration date"
            ),
        ),
    ]
