# Generated by Django 3.2.7 on 2021-10-03 15:34

from django.db import migrations

import ephios.core.models.users


class Migration(migrations.Migration):

    dependencies = [
        ("eventautoqualification", "0003_alter_eventautoqualificationconfiguration_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventautoqualificationconfiguration",
            name="expiration_date",
            field=ephios.core.models.users.ExpirationDateField(
                blank=True, null=True, verbose_name="Expiration date"
            ),
        ),
    ]
