{% extends "base.html" %}
{% load crispy_forms_filters %}
{% load i18n %}

{% block title %}
    {% trans "Login" %}
{% endblock %}

{% block content %}
    {% trans "Login" as login_message %}
    <h2>{{ login_message }}</h2>
    <form method="post">
        {% csrf_token %}
        {{ form|crispy }}
        <button class="btn btn-primary" type="submit">{{ login_message }}</button>
        <a class="float-end" href="{% url "password_reset" %}">{% translate "Forgot your password?" %}</a>
    </form>
{% endblock %}