{% extends "base.html" %}
{% load i18n %}

{% block title %}
    {% translate "Delete events" %}
{% endblock %}

{% block content %}
    <div class="page-header">
        <h1>{% translate "Delete events" %}</h1>
    </div>
    <p>{% translate "Are you sure that you want to delete the following events?" %}</p>
    <form method="post">
        {% csrf_token %}
        <input type="hidden" name="confirm" value="true">
        <ul>
            {% for event in events %}
                <input type="hidden" name="bulk_action" value="{{ event.pk }}">
                <li>{{ event.title }} ({{ event.get_start_time }} - {{ event.get_end_time }})</li>
            {% endfor %}
        </ul>
        <a role="button" class="btn btn-secondary"
           href="{% url "core:event_list" %}">{% trans "Back" %}</a>
        <button type="submit" class="btn btn-danger">{% trans "Delete" %}</button>
    </form>
{% endblock %}