{% extends "core/settings/settings_base.html" %}
{% load crispy_forms_filters %}
{% load i18n %}

{% block settings_content %}
    <form method="post">
     {% csrf_token %}
    {{ form|crispy }}
    <button class="btn btn-primary" type="submit">{% translate "Save" %}</button>
    </form>
{% endblock %}
