{% extends "base.html" %}
{% load webpush_notifications %}
{% load webpush_notifications %}
{% load utils %}
{% load crispy_forms_filters %}
{% load i18n %}

{% block title %}
    {% translate "Settings" %}
{% endblock %}

{% block html_head %}
    {% webpush_header %}
{% endblock %}

{% block content %}
    <div class="page-header">
        <h1>{% trans "Notifications" %}</h1>
    </div>

    <p>{% translate "You can configure whether you want to receive notifications for the following occasions:" %}</p>
    <form method="post">
        {% csrf_token %}
        {{ form|crispy }}
        <button class="btn btn-primary">{% translate "Save" %}</button>
    </form>

    <h3 class="mt-4">{% translate "Push notifications" %}</h3>
    <p>
        {% translate "You can manage the occasions you want to receive push notificaions for above. You can manage if you want to receive push notifications on this device at all below." %}
    </p>
    {% webpush_button with_class="btn btn-sm btn-outline-primary" %}

{% endblock %}
