<div class="row align-items-center">
    {% spaceless %}{% for widget in widget.subwidgets %}
        <div class="col"> {% include widget.template_name %} </div>
    {% endfor %}{% endspaceless %}
</div>