# Generated by Django 3.1.7 on 2021-03-23 22:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0006_notification"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventtype",
            name="color",
            field=models.CharField(default="#343a40", max_length=7, verbose_name="color"),
        ),
    ]
