# Generated by Django 3.1.5 on 2021-03-05 17:55

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("core", "0003_migrate_content_type_to_core_20210219_1642"),
    ]

    operations = [
        migrations.AlterField(
            model_name="abstractparticipation",
            name="polymorphic_ctype",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="polymorphic_core.abstractparticipation_set+",
                to="contenttypes.contenttype",
            ),
        ),
        migrations.AlterField(
            model_name="abstractparticipation",
            name="shift",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="participations",
                to="core.shift",
                verbose_name="shift",
            ),
        ),
    ]
