{% load i18n %}

<div class="mb-3">
    {% for participation in participations %}
        {% include "basesignup/participation_card_inline.html" %}
    {% endfor %}
    {% for index in empty_spots %}
        <span class="badge bg-light px-5">&nbsp;</span>
    {% endfor %}
</div>

{% with stats=shift.signup_method.get_signup_stats %}
    <span class="pe-1">
        <strong>{{ stats.confirmed_count }}{% if stats.max_count %}/{{ stats.max_count }}{% endif %}</strong>
        {% translate "Total" context "participation count" %}
    </span>
{% endwith %}

{% if disposition_url %}
    <div class="mt-1">
        <a href="{{ disposition_url }}" class="btn btn-sm btn-info">{% translate "Disposition" %}</a>
    </div>
{% endif %}