// 1. Include functions first (so you can manipulate colors, SVGs, calc, etc)
@import "../../bootstrap/scss/functions";

// 2. Include any default variable overrides here
@import "ephios_bootstrap_variables";

// 3. Include bootstrap
@import "../../bootstrap/scss/variables";
@import "../../bootstrap/scss/mixins";
@import "../../bootstrap/scss/utilities";

// Layout & components
@import "../../bootstrap/scss/root";
@import "../../bootstrap/scss/reboot";
@import "../../bootstrap/scss/type";
@import "../../bootstrap/scss/images";
@import "../../bootstrap/scss/containers";
@import "../../bootstrap/scss/grid";
@import "../../bootstrap/scss/tables";
@import "../../bootstrap/scss/forms";
@import "../../bootstrap/scss/buttons";
@import "../../bootstrap/scss/transitions";
@import "../../bootstrap/scss/dropdown";
@import "../../bootstrap/scss/button-group";
@import "../../bootstrap/scss/nav";
@import "../../bootstrap/scss/navbar";
@import "../../bootstrap/scss/card";
@import "../../bootstrap/scss/accordion";
@import "../../bootstrap/scss/breadcrumb";
@import "../../bootstrap/scss/pagination";
@import "../../bootstrap/scss/badge";
@import "../../bootstrap/scss/alert";
@import "../../bootstrap/scss/progress";
@import "../../bootstrap/scss/list-group";
@import "../../bootstrap/scss/close";
@import "../../bootstrap/scss/toasts";
@import "../../bootstrap/scss/modal";
@import "../../bootstrap/scss/tooltip";
@import "../../bootstrap/scss/popover";
@import "../../bootstrap/scss/carousel";
@import "../../bootstrap/scss/spinners";
@import "../../bootstrap/scss/offcanvas";

// Helpers and utilities
@import "../../bootstrap/scss/helpers";
@import "../../bootstrap/scss/utilities/api";

// 4. Add additional custom code here
@import "ephios_custom";