{% extends "core/settings/settings_base.html" %}
{% load i18n %}
{% load crispy_forms_filters %}


{% block settings_content %}
    <h2>
    {% translate "Qualification import" %}
    </h2>
    {% crispy form %}
{% endblock %}
