<div class="list-group-item draggable" data-formset-form data-participant-id="{{ form.instance.user.id }}">
    {% block participation_body %}

    {% endblock %}
    <div class="d-none">
        {{ form.DELETE }}
        {{ form.state }}
        {{ form.id }}
    </div>
</div>
