{% extends "core/settings/settings_base.html" %}
{% load i18n %}

{% block settings_content %}
    <a class="btn btn-secondary" href="{% url "core:settings_eventtype_create" %}"><span
                class="fa fa-plus"></span> {% trans "Add event type" %}</a>
    <table id="event_table" class="table table-striped display mt-2">
        <thead>
        <tr>
            <th>{% translate "Title" %}</th>
            <th>{% translate "Action" %}</th>
        </tr>
        </thead>
        <tbody>
        {% for eventtype in eventtype_list %}
            <tr>
                <td class="break-word">{{ eventtype.title }}</td>
                <td class="d-flex">
                    <a class="btn btn-secondary btn-sm text-nowrap"
                       href="{% url "core:settings_eventtype_edit" eventtype.pk %}"><span
                            class="fa fa-edit"></span> <span class="d-none d-md-inline">{% trans "Edit" %}</span></a>
                    <a class="btn btn-secondary btn-sm text-nowrap ms-1"
                       href="{% url "core:settings_eventtype_delete" eventtype.pk %}"><span
                            class="fa fa-trash-alt"></span> <span class="d-none d-md-inline">{% trans "Delete" %}</span></a>
                </td>
            </tr>
        {% endfor %}
        </tbody>
    </table>
{% endblock %}
