{% extends "base.html" %}
{% load i18n %}

{% block title %}
    {% trans "Logged out" %}
{% endblock %}

{% block content %}
    <p>{% trans "Logged out successfully." %}</p>
    <a class="btn btn-primary" href="{% url "login" %}">{% trans "Back to login" %}</a>
{% endblock %}
