# Generated by Django 3.1.7 on 2021-04-03 21:48

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0007_auto_20210324_2101"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="consequence",
            options={"verbose_name": "Consequence"},
        ),
        migrations.AlterModelOptions(
            name="qualificationgrant",
            options={"verbose_name": "Qualification grant"},
        ),
        migrations.RemoveField(
            model_name="consequence",
            name="decided_by",
        ),
        migrations.RemoveField(
            model_name="consequence",
            name="executed_at",
        ),
        migrations.RemoveField(
            model_name="consequence",
            name="fail_reason",
        ),
        migrations.AlterField(
            model_name="abstractparticipation",
            name="finished",
            field=models.BooleanField(default=False, verbose_name="finished"),
        ),
        migrations.AlterField(
            model_name="consequence",
            name="state",
            field=models.TextField(
                choices=[
                    ("needs_confirmation", "needs confirmation"),
                    ("executed", "executed"),
                    ("failed", "failed"),
                    ("denied", "denied"),
                ],
                default="needs_confirmation",
                max_length=31,
                verbose_name="State",
            ),
        ),
        migrations.AlterField(
            model_name="localparticipation",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="Participant",
            ),
        ),
        migrations.AlterField(
            model_name="shift",
            name="event",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="shifts",
                to="core.event",
                verbose_name="event",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="is_active",
            field=models.BooleanField(default=True, verbose_name="Active"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="is_staff",
            field=models.BooleanField(default=False, verbose_name="Staff user"),
        ),
    ]
