{% load i18n %}

<div>
    {% for section in sections.values %}
        <span class="pe-1">
            <strong>
            {{ section.stats.confirmed_count }}{% if section.stats.max_count %}/{{ section.stats.max_count }}{% endif %}
        </strong>
            {{ section.title }}
            <span class="text-muted float-end">{{ section.qualifications_label }}</span>
        </span>

        <div class="mb-4">
            {% for participation in section.participations %}
                {% include "basesignup/participation_card_inline.html" %}
            {% endfor %}
            {% for index in section.placeholder %}
                <span class="badge bg-light px-5">&nbsp;</span>
            {% endfor %}
            {% if not section.participations and not section.placeholder %}
                {# show a white box as vertical whitespace  #}
                <span class="badge bg-white px-5">&nbsp;</span>
            {% endif %}
        </div>
    {% endfor %}
</div>

{% with stats=shift.signup_method.get_signup_stats %}
    <span class="pe-1">
        <strong>{{ stats.confirmed_count }}{% if stats.max_count %}/{{ stats.max_count }}{% endif %}</strong>
        {% translate "Total" context "participation count" %}
    </span>
{% endwith %}

{% if disposition_url %}
    <div class="mt-1">
        <a href="{{ disposition_url }}" class="btn btn-sm btn-info">{% translate "Disposition" %}</a>
    </div>
{% endif %}
